<?php
/**
 * The plugin bootstrap file
 *
 * This file is read by WordPress to generate the plugin information in the plugin
 * admin area. This file also includes all of the dependencies used by the plugin,
 * registers the activation and deactivation functions, and defines a function
 * that starts the plugin.
 *
 * @link              http://www.centralpay.eu/
 * @since             1.0.0
 * @package           cpay
 *
 * @wordpress-plugin
 * Plugin Name:       CentralPay
 * Plugin URI:        http://wordpress.org/plugins/cpay/
 * Description:       Réalisez des paiements par carte de crédit avec CentralPay.
 * Version:           1.1.1
 * Author:            CentralPay
 * Author URI:        http://www.centralpay.eu/
 * License:           GPL-2.0+
 * License URI:       http://www.gnu.org/licenses/gpl-2.0.txt
 * Text Domain:       cpay
 * Domain Path:       /languages
 * Slug:              cpay
 */

// If this file is called directly, abort.
if (! defined( 'WPINC' )) {
    die;
}

/*
* Check if WooCommerce is active
*/
if (in_array('woocommerce/woocommerce.php', apply_filters('active_plugins', get_option('active_plugins')))) {

/**
 * The code that runs during plugin activation.
 * This action is documented in includes/class-cpay-activator.php
 */
    function activateCpay()
    {
        require_once plugin_dir_path( __FILE__ ) . 'includes/class-cpay-activator.php';
        CpayActivator::activate();
    }

/**
 * The code that runs during plugin deactivation.
 * This action is documented in includes/class-cpay-deactivator.php
 */
    function deactivateCpay()
    {
        require_once plugin_dir_path( __FILE__ ) . 'includes/class-cpay-deactivator.php';
        CpayDeactivator::deactivate();
    }

    /**
     * Account menu items
     *
     * @param arr $items
     * @return arr
     */
    function iconic_account_menu_items($items)
    {
        if (isset($items)) {
            $items['cardedit'] = __( 'Card Edit', 'iconic' );
            
            return $items;
        } else {
            throw new Exception("An error was occurred when the iconic_account_menu_items was called beacause the argument is not exist.");
        }
    }

    add_filter( 'woocommerce_account_menu_items', 'iconic_account_menu_items', 10, 1 );

    /**
     * Add endpoint
     */
    function iconic_add_my_account_endpoint()
    {
        add_rewrite_endpoint( 'cardedit', EP_PAGES );
    }

    add_action( 'init', 'iconic_add_my_account_endpoint' );


     require plugin_dir_path( __FILE__ ) . 'includes/class-cpay-wp-db.php';

    /**
     * cardedit content
     */
    function iconic_cardedit_endpoint_content()
    {
        try {
            $current_user = wp_get_current_user( );
            $wpUserId = (int)$current_user->ID;
            $cards = (object)DB::getCards($wpUserId, DB_USER, DB_PASSWORD);
            $html = '<div id="center_column" class="center_column col-xs-12 col-sm-12">
                        <h1 class="page-heading bottom-indent">My credit cards</h1>
                        <div class="block-center" id="block-history">
                                    <table id="order-list" class="table table-bordered footab">
                                    <thead>
                                    <tr>
                                        <th class="first_item" data-sort-ignore="true">Numéro de carte</th>
                                        <th class="item">Type carte</th>
                                        <th class="item">Date d\'expiraton</th>
                                        <th class="item">Défaut</th>
                                        <th class="item">Action</th>
                                    </tr>
                                    </thead>
                                    <tbody>';
            foreach ($cards as $key => $row) {
                $html .='<tr class="first_item ">
                                        <td>'.$row->first6.'---------'.$row->last4.'</td>
                                        <td>';
                                        $firstChar = substr($row->first6, 0, -5);
                if ($firstChar === '4') {
                    $html .= 'Visa';
                }
                if ($firstChar === '5') {
                    $html .= 'Mastercard';
                }
                // var_dump($row);die();
                                            $html .= '</td>';
                                            $html .= '<td>'.$row->expirationMonth.'/'.$row->expirationYear.'</td>';
                                            $html .= '<td>';
                                            $isDefault = ($row->default == true) ? 'Yes' : 'No';
                                            $html .= $isDefault;
                                            '</td>';
                                            $html .= '<td><a class="btn btn-default button button-small" href="'
                                                . plugin_dir_url(__FILE__)
                                                . 'includes/class-cpay-managed-card.php?action=delete&amp;card='
                                                . $row->cardId
                                                . '&amp;dbname='
                                                . DB_NAME
                                                . '" data-hasqtip="22" oldtitle="Supprimer" title="">
                                                    <span>Supprimer</span>
                                                </a>&nbsp;&nbsp;
                                                <a class="link-button" href="' 
                                                . plugin_dir_url(__FILE__)
                                                . 'includes/class-cpay-managed-card.php?action=default&amp;card='
                                                . $row->cardId
                                                .'&amp;dbname='
                                                . DB_NAME
                                                .'" data-hasqtip="23" oldtitle="Mettre par défaut" title="">
                                                    <i class="icon-refresh"></i>Défaut
                                                </a>
                                            </td>';
                                            // var_dump($row->default);die();
                                        $html .='</tr>';
            }
                                                $html .='</tbody></table>
                                <div id="block-order-detail" class="unvisible">&nbsp;</div>
                            </div>
                        <ul class="footer_links clearfix">
                            <li>
                                <a class="btn btn-default button button-small" href="http://wpcpay.fdo.loc/mon-compte/">
                                    <span>
                                        <i class="icon-chevron-left"></i> Back to Your Account
                                    </span>
                                </a>
                            </li>
                            <li>
                                <a class="btn btn-default button button-small" href="http://wpcpay.fdo.loc/">
                                    <span><i class="icon-chevron-left"></i> Home</span>
                                </a>
                            </li>
                        </ul>
					</div>';
            echo $html;
        } catch (Exception $e) {
            throw new Exception($e->getMessage());
        }
    }


    add_action( 'woocommerce_account_cardedit_endpoint', 'iconic_cardedit_endpoint_content' );


    /**
     * Helper: is endpoint
     */
    function iconic_is_endpoint($endpoint = false)
    {

        global $wp_query;

        if (!$wp_query) {
            return false;
        }

        return isset( $wp_query->query[ $endpoint ] );
    }
    
    function wpb_woo_my_account_order()
    {
        $myorder = array(
            'dashboard' => __( 'Dashboard', 'woocommerce' ),
            'cardedit' => __( 'Card Edit', 'woocommerce' ),
            'orders' => __( 'Orders', 'woocommerce' ),
            'downloads' => __( 'Downloads', 'woocommerce' ),
            'edit-address' => __( 'Addresses', 'woocommerce' ),
            'edit-account' => __( 'Account details', 'woocommerce' ),
            'customer-logout' => __( 'Logout', 'woocommerce' ),
           );
           return $myorder;
    }
    add_filter ( 'woocommerce_account_menu_items', 'wpb_woo_my_account_order' );

    register_activation_hook( __FILE__, 'activateCpay' );
    register_deactivation_hook( __FILE__, 'deactivateCpay' );

/**
 * The core plugin class that is used to define internationalization,
 * admin-specific hooks, and public-facing site hooks.
 */
     require plugin_dir_path( __FILE__ ) . 'includes/class-cpay.php';
 
/**
 * Begins execution of the plugin.
 *
 * Since everything within the plugin is registered via hooks,
 * then kicking off the plugin from this pooint in the file does
 * not affect the page life cycle.
 *
 * @since    1.0.0visua
 */
    function main()
    {
         $plugin = new Cpay();
         // Run the Cpay plugin
         $plugin->run();
    }

    main();
}
