<?php
if (!defined("ABSPATH")) {
    exit;
}

/**
*
* Give access to the CentralPay API, you can do request on this.
* @since    1.0.0
*/
class CpayApi
{

    /**
     * Send a POST request using cURL
     * @param string $url to request
     * @param array $post values to send
     * @param array $options for cURL
     * @return array column in the array ['header','body']
     */
    public static function curl_post($url, $userpwd, array $post = null)
    {
        if (!isset($url) or !isset($userpwd)) {
            throw new Exception("An error was occured when the CURl POST has called, a parameters was empty.");
        } else {
            try {
                $defaults = array(
                    CURLOPT_URL => $url,
                    CURLOPT_RETURNTRANSFER => 1,
                    CURLOPT_POSTFIELDS => http_build_query($post),
                    CURLOPT_POST => 1,
                    CURLOPT_SSL_VERIFYPEER => false,
                    CURLOPT_HEADER => 1,
                    CURLOPT_USERPWD => $userpwd
                );

                $curl = curl_init();
                curl_setopt_array($curl, $defaults);

                $result = curl_exec($curl);

                $info = curl_getinfo($curl);
                $header_size = curl_getinfo($curl, CURLINFO_HEADER_SIZE);

                curl_close($curl);

                if (empty($info['http_code'])) {
                    die("No HTTP code was returned");
                }

                $body = substr($result, $header_size);

                return array(
                'http_code' => $info['http_code'],
                'body' => $body
                );
            } catch (Exception $e) {
                throw new Exception($e->getMessage());
            }
        }
    }

    /**
     * Send a GET requst using cURL
     * @param string $url to request
     * @param array $get values to send
     * @param array $options for cURL
     * @return string
     */
    public static function curl_get($url, $userpwd, array $post = null)
    {
        if (!isset($url) or !isset($userpwd)) {
            throw new Exception("An error was occured when the CURl POST has called, a parameters was empty.");
        } else {
            try {
                $defaults = array(
                    CURLOPT_URL => $url. (strpos($url, '?') === false ? '?' : ''). http_build_query($post),
                    CURLOPT_HEADER => 0,
                    CURLOPT_RETURNTRANSFER => true,
                    CURLOPT_TIMEOUT => 4
                    );

                $curl = curl_init();
                curl_setopt_array($curl, $defaults);

                $result = curl_exec($curl);

                $info = curl_getinfo($curl);
                $header_size = curl_getinfo($curl, CURLINFO_HEADER_SIZE);

                curl_close($curl);

                if (empty($info['http_code'])) {
                    die("No HTTP code was returned");
                }

                $body = substr($result, $header_size);

                return array(
                'http_code' => $info['http_code'],
                'body' => $body
                );
            } catch (Exception $e) {
                throw new Exception($e->getMessage());
            }
        }
    }
}
