<?php
if (!defined('ABSPATH')) {
    exit;
}

/**
 *
 * Data access to the Wordpress database
 *
 * @since    1.0.0
 */
class DB
{

    /**
     *
     * Allow to create a customer in wordpress data base
     * @param $transaction This array take the transaction information
     *
     * @since    1.0.0
     *
     */
    public static function createCustomer($transactionData, $dbuser, $dbpassword)
    {
        if (!isset($transactionData) or !isset($dbuser) or !isset($dbpassword)) {
            throw new Exception("An error was occurred when the db create customer was called, one of this parameters has not set : transactionData or dbuser or dbpassword.");
        } else {
            try {
                global $wpdb;

                $wpdb->insert($wpdb->prefix.'woocommerce_cpay_customer', array(
                    'cpayCustomerId' => $transactionData['customerId'],
                    'wpUserId' => $transactionData['userId']
                ));
            } catch (PDOException $e) {
                throw new PDOException('Connexion échouée : ' . $e->getMessage());
            } catch (Exception $e) {
                throw new Exception($e->getMessage());
            }
        }
    }

    /**
     *
     *  Allow to create a card in wordpress data base.
     *
     * @since    1.0.0
     * @param $transaction This array take the transaction information
     *
     */
    public static function createCard($transactionData, $dbuser, $dbpassword)
    {
        global $wpdb;

        if (!isset($transactionData) or !isset($dbuser) or !isset($dbpassword)) {
            throw new Exception("An error was occurred when the createCard function was called, one of his parameters has not exist : transactionData or dbuser or dbpassword.");
        } else {
            try {

                $wpdb->insert($wpdb->prefix.'woocommerce_cpay_card', array(
                    'customerId' => $transactionData['customerId'],
                    'cardId' => $transactionData['cardId'],
                    'first6' => $transactionData['first6'],
                    'last4' => $transactionData['last4'],
                    'expirationYear' => $transactionData['expirationYear'],
                    'expirationMonth' => $transactionData['expirationMonth'],
                    'default' => 'false'
                ));

            } catch (Exception $exception) {
                throw new Exception($exception->getMessage());
            }
        }
    }

    /**
     * This function allow you to get the customer ID in the wordpress db
     *
     * @param integer $userId
     * @return customerId Integer type or false
     */
    public static function getCustomerId($userId, $dbuser, $dbpassword)
    {
        if (!isset($userId) or !isset($dbuser) or !isset($dbpassword)) {
            throw new Exception("An error was occurred when the get customer id function was called, one of his parameters has not exist : userId or dbuser or dbpassword.");
        } else {
            try {
                global $wpdb;
                $row = $wpdb->get_row("SELECT cpayCustomerId FROM ".$wpdb->prefix."woocommerce_cpay_customer WHERE wpUserId = $userId", ARRAY_A);
                return ($row !== null) ? $row['cpayCustomerId'] : false;
            } catch (Exception $e) {
                throw new Exception($e->getMessage( ));
            }
        }
    }

    /**
     * Allow you to get the cards with the wordpress user id
     *
     * @param Integer $wpUserId
     * @return Cards
     */
    public static function getCards($wpUserId, $dbUser, $dbPassword)
    {
        if (isset($wpUserId) == false or isset($dbUser) == false or isset($dbPassword)  == false) {
            throw new Exception("An error has occurred when the getCards function has called because an argument below are not exist : wpUserId, dbUser, dbPassword.");
        } else {
            try {
                if (!is_numeric($wpUserId)) {
                    throw new InvalidArgumentException("An exception are occurred when you have try to get the cards because the wpUserid is not an integer or string in integer.");
                } elseif (!is_string($dbUser) || !is_string($dbPassword)) {
                    throw new InvalidArgumentException("An exception are occurred when you have try to get the cards because the dbUser or dbpassword is not string.");
                } else {
                    global $wpdb;

                    $rows = $wpdb->get_results("SELECT card.cardId, card.first6, card.last4, card.expirationMonth, card.expirationYear, card.default
                                        FROM ".$wpdb->prefix."woocommerce_cpay_card As card 
                                        INNER JOIN ".$wpdb->prefix."woocommerce_cpay_customer AS customer ON card.customerId = customer.cpayCustomerId 
                                        WHERE customer.wpUserId = $wpUserId");

                    return $rows;
                }
            } catch (Exception $e) {
                throw new Exception($e->getMessage());
            }
        }
    }

    /**
     * This function allow you to delete a card in the wordpress data base
     *  DBname is use in parameter because this function is use outside of the woocommerce environment (I need to get the good db name)
     * @param int $cardid
     * @param string $dbUser
     * @param string $dbpassword
     * @return void
     */
    public static function deleteCard($cardid, $dbname)
    {
        if (!isset($cardid) || !isset($dbname)) {
            throw new Exception("An error has occurred when you have delete the card, one of this parameters : cardid, dbUser or dbPassword is not set.");
        } else {
            try {
                global $wpdb;
                $wpdb->delete($wpdb->prefix.'woocommerce_cpay_card', array( 'cardId' => $cardid ), array( '%s' ) );
                if ($wpdb->last_error == '') {
                    wp_safe_redirect(wp_get_referer());
                } else {
                    throw new Exception("An error was occurred when you have try to delete a card.");
                }
            } catch (Exception $e) {
                throw new Exception($e->getMessage());
            }
        }
    }


    /**
     * This function allow you to set a default payment card for a user.
     *
     * @param int $cardid
     * @param string $default
     * @param string $dbname
     * @return void
     */
    public static function setDefaultCard($cardid, $dbname)
    {
        if (!isset($cardid) or !isset($dbname)) {
            throw new Exception("Une erreur est survenue lors de la sélection de la carte de paiement par défaut, l'un des paramètres suivants : cardid, dbUser ou dbname est inexistant.");
        } else {
            try {
                // var_dump($dbname.' dbname and cardid '.$cardid);die();
                self::deactivateDefaultCard(DB_USER, DB_PASSWORD, $dbname);
                global $wpdb ;
                $wpdb->update($wpdb->prefix.'woocommerce_cpay_card', array('default' => '1'), array('cardId' => $cardid));
                if ($wpdb->last_error == '') {
                    wp_safe_redirect(wp_get_referer());
                } else {
                    throw new Exception("An error was occurred when you have try to apply a default card.");
                }
            } catch (Exception $e) {
                throw new Exception($e->getMessage());
            }
        }
    }

    /**
     * This function allow you to deactivate a default card
     *
     * @param string $dbUser
     * @param string $dbPassword
     * @return void
     */
    public static function deactivateDefaultCard($dbUser, $dbPassword, $dbname)
    {
        if (!isset($dbUser) or !isset($dbPassword) or !isset($dbname)) {
            throw new Exception("An error was occurred when the deactivate default card was called because a parameter as not set (db user or db password or db name)");
        } else {
            try {
                global $wpdb ;
                $wpdb->update($wpdb->prefix.'woocommerce_cpay_card', array('default' => '0'), array('default' => '1'));
                if ($wpdb->last_error != '') {
                    throw new Exception("An error was occurred when you have try to deactivate the old default card.");
                }
            } catch (Exception $e) {
                throw new Exception($e->getMessage());
            }
        }
    }

    /**
     * This function allow you to create a transaction. That insert in db allow you to bind cpay card transaction with centralpay api.
     *
     * @param array $data
     * @param string $dbuser
     * @param string $dbpassword
     * @return void
     */
    public static function createTransaction($data, $dbuser, $dbpassword) {
        if (!isset($dbUser) or !isset($dbpassword)) {
            throw new Exception("An error was occurred when the create transaction function was called because a parameters is not set.");
        }else {
            try{
                global $wpdb ;
                $wpdb->insert($wpdb->prefix.'woocommerce_cpay_transactions_card', array(
                    'cardTransactionId' => $data['cardTransactionId'],
                    'cardId' => $data['cardId']
                ));
            }catch(Exception $e) {
                throw new Exception($e->getMessage());
            }
        }
    }
}
