<?php
if (! defined('ABSPATH')) {
    exit;
}

class WCGatewayCpay extends WC_Payment_Gateway_CC
{
    /**
     * The database name
     *
     * @var string
     */
    private $_dbname;

    /**
     * he data base host
     *
     * @var string
     */
    private $_dbhost;

    /**
     * The data base user
     *
     * @var string
     */
    private $_dbuser;

    /**
     * The data base password
     *
     * @var string
     */
    private $_dbpassword;

    /**
     * Get the test mode settings
     *
     * @var string
     */
    private $testmode;

    /**
     * Get the _paReq value
     *
     * @var string
     */
    private $_paReq;

    /**
     * Get the _acsURL value
     *
     * @var string
     */
    private $_acsURL;

    /**
     * Get the termUrl
     *
     * @var string
     */
    private $_termUrl;

    /**
     * The payment data variable.
     *
     * @var Array
     */
    private $_paymentData;

    /**
     * The postvars for the transaction
     *
     * @var Array
     */
    private $_postvars;

    /*
     * The selection of payment popin
     */
    private $popin;

    /**
     * The wordpress langage
     *
     * @var string
     */
    private $langage;

    /**
     * The debug mode
     *
     * @var bool
     */
    private $debug;

    /**
     *
     * Initialize the parents members and the plugin parameters.
     * @since    1.0.0
     */
    public function __construct()
    {
        if (! class_exists( 'CpayConfig' )) {
            require plugin_dir_path( __FILE__ ) . 'class-cpay-config.php';
        }
        $this->langage = get_locale();
        $this->id                      = 'cpay';
        $this->method_title            = __( 'CentralPay', 'cpay' );
        $this->method_description      = __( 'CentralPay allow you to pay online.', 'cpay' );
        $this->has_fields              = true;
        $this->order_button_text       = __( 'Proceed to payment', 'cpay' );
        $this->title                   = __( 'Payed with card', 'cpay' );
        $this->init_form_fields();
        $this->init_settings();
        // Afficher la description du mode test
        if ($this->get_option( 'mode_test' ) === 'yes') {
            $this->url_cpay                = CpayConfig::CPAY_API_TEST_URI;
            $this->url_js                  = CpayConfig::CPAY_JS_TEST_URI;
        } else {
            $this->url_cpay                = CpayConfig::CPAY_API_PROD_URI;
            $this->url_js                  = CpayConfig::CPAY_JS_PROD_URI;
        }
        $this->login_cpay              = $this->get_option( 'login_cpay' );
        $this->password_cpay           = $this->get_option( 'password_cpay' );
        $this->merchant_public_key     = $this->get_option( 'merchant_public_key' );
        $this->point_of_sale_id        = $this->get_option( 'point_of_sale_id' );
        $this->testmode                = ( ! empty( $this->get_option( 'mode_test' ) ) && 'yes' === $this->get_option( 'mode_test' ) ) ? true : false;
        $this->_3ds                    = ( ! empty( $this->get_option( '_3ds' ) ) && 'yes' === $this->get_option( '_3ds' ) ) ? true : false;
        // POPIN : Décommenter si besoin
        // $this->popin                   = $this->get_option( 'popin' );
        $this->debug                   = false;
        $this->_dbname                 = DB_NAME;
        $this->_dbhost                 = DB_HOST;
        $this->_dbuser                 = DB_USER;
        $this->_dbpassword             = DB_PASSWORD;
        $this->_paymentData            = array();
        if (! class_exists( 'DB' )) {
            require plugin_dir_path( __FILE__ ) . 'class-cpay-wp-db.php';
        }
        if (! class_exists( 'CpayApi' )) {
            require plugin_dir_path( __FILE__ ) . 'class-cpay-api.php';
        }
        // Custom Credit Card Icons
        add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array( $this, 'process_admin_options' ) );
    }

    /**
     *
     * Initialise Gateway Settings Form Fields
     *
     * @since    1.0.0
     */
    public function init_form_fields()
    {
        $this->form_fields = include( 'settings-cpay.php' );
    }

    /**
     * get_icon function.
     *
     * @access public
     * @return string
     */
    public function get_icon()
    {
        // VISA
        $icon  = '<div class="label-image">
                    <svg width="27" height="27" version="1.1" id="Layer_1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" x="0px" y="0px"
                                viewBox="10 0 504 504" style="enable-background:new 0 0 504 504;" xml:space="preserve">
                            <polygon style="fill:#3C58BF;" points="184.8,324.4 210.4,180.4 250.4,180.4 225.6,324.4 "/>
                            <polygon style="fill:#293688;" points="184.8,324.4 217.6,180.4 250.4,180.4 225.6,324.4 "/>
                            <path style="fill:#3C58BF;" d="M370.4,182c-8-3.2-20.8-6.4-36.8-6.4c-40,0-68.8,20-68.8,48.8c0,21.6,20,32.8,36,40
                                s20.8,12,20.8,18.4c0,9.6-12.8,14.4-24,14.4c-16,0-24.8-2.4-38.4-8l-5.6-2.4l-5.6,32.8c9.6,4,27.2,8,45.6,8
                                c42.4,0,70.4-20,70.4-50.4c0-16.8-10.4-29.6-34.4-40c-14.4-7.2-23.2-11.2-23.2-18.4c0-6.4,7.2-12.8,23.2-12.8
                                c13.6,0,23.2,2.4,30.4,5.6l4,1.6L370.4,182L370.4,182z"/>
                            <path style="fill:#293688;" d="M370.4,182c-8-3.2-20.8-6.4-36.8-6.4c-40,0-61.6,20-61.6,48.8c0,21.6,12.8,32.8,28.8,40
                                s20.8,12,20.8,18.4c0,9.6-12.8,14.4-24,14.4c-16,0-24.8-2.4-38.4-8l-5.6-2.4l-5.6,32.8c9.6,4,27.2,8,45.6,8
                                c42.4,0,70.4-20,70.4-50.4c0-16.8-10.4-29.6-34.4-40c-14.4-7.2-23.2-11.2-23.2-18.4c0-6.4,7.2-12.8,23.2-12.8
                                c13.6,0,23.2,2.4,30.4,5.6l4,1.6L370.4,182L370.4,182z"/>
                            <path style="fill:#3C58BF;" d="M439.2,180.4c-9.6,0-16.8,0.8-20.8,10.4l-60,133.6h43.2l8-24h51.2l4.8,24H504l-33.6-144H439.2z
                                M420.8,276.4c2.4-7.2,16-42.4,16-42.4s3.2-8.8,5.6-14.4l2.4,13.6c0,0,8,36,9.6,44h-33.6V276.4z"/>
                            <path style="fill:#293688;" d="M448.8,180.4c-9.6,0-16.8,0.8-20.8,10.4l-69.6,133.6h43.2l8-24h51.2l4.8,24H504l-33.6-144H448.8z
                                M420.8,276.4c3.2-8,16-42.4,16-42.4s3.2-8.8,5.6-14.4l2.4,13.6c0,0,8,36,9.6,44h-33.6V276.4z"/>
                            <path style="fill:#3C58BF;" d="M111.2,281.2l-4-20.8c-7.2-24-30.4-50.4-56-63.2l36,128h43.2l64.8-144H152L111.2,281.2z"/>
                            <path style="fill:#293688;" d="M111.2,281.2l-4-20.8c-7.2-24-30.4-50.4-56-63.2l36,128h43.2l64.8-144H160L111.2,281.2z"/>
                            <path style="fill:#FFBC00;" d="M0,180.4l7.2,1.6c51.2,12,86.4,42.4,100,78.4l-14.4-68c-2.4-9.6-9.6-12-18.4-12H0z"/>
                            <path style="fill:#F7981D;" d="M0,180.4L0,180.4c51.2,12,93.6,43.2,107.2,79.2l-13.6-56.8c-2.4-9.6-10.4-15.2-19.2-15.2L0,180.4z"/>
                            <path style="fill:#ED7C00;" d="M0,180.4L0,180.4c51.2,12,93.6,43.2,107.2,79.2l-9.6-31.2c-2.4-9.6-5.6-19.2-16.8-23.2L0,180.4z"/>
                            <g>
                                <path style="fill:#051244;" d="M151.2,276.4L124,249.2l-12.8,30.4l-3.2-20c-7.2-24-30.4-50.4-56-63.2l36,128h43.2L151.2,276.4z"/>
                                <polygon style="fill:#051244;" points="225.6,324.4 191.2,289.2 184.8,324.4 	"/>
                                <path style="fill:#051244;" d="M317.6,274.8L317.6,274.8c3.2,3.2,4.8,5.6,4,8.8c0,9.6-12.8,14.4-24,14.4c-16,0-24.8-2.4-38.4-8
                                    l-5.6-2.4l-5.6,32.8c9.6,4,27.2,8,45.6,8c25.6,0,46.4-7.2,58.4-20L317.6,274.8z"/>
                                <path style="fill:#051244;" d="M364,324.4h37.6l8-24h51.2l4.8,24H504L490.4,266l-48-46.4l2.4,12.8c0,0,8,36,9.6,44h-33.6
                                    c3.2-8,16-42.4,16-42.4s3.2-8.8,5.6-14.4"/>
                            </g>
                            <g>
                            </g>
                            <g>
                            </g>
                            <g>
                            </g>
                            <g>
                            </g>
                            <g>
                            </g>
                            <g>
                            </g>
                            <g>
                            </g>
                            <g>
                            </g>
                            <g>
                            </g>
                            <g>
                            </g>
                            <g>
                            </g>
                            <g>
                            </g>
                            <g>
                            </g>
                            <g>
                            </g>
                            <g>
                            </g>
                            </svg>
                    ';

        // CB
        $icon .= '<svg width="27" height="27" version="1.1" id="Calque_1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" x="0px" y="0px"
                        viewBox="0 0 60 60" style="enable-background:new 0 0 60 60;" xml:space="preserve">
                        <style type="text/css">
                                        .VoÃ_x00BB_teÂ_x00A0__x002D__x0020_Vert{fill:url(#SVGID_1_);stroke:#FFFFFF;stroke-width:0.25;stroke-miterlimit:1;}
                                        .st0{fill:url(#SVGID_2_);}
                                        .st1{fill:#FFFFFF;}
                        </style>
                        <linearGradient id="SVGID_1_" gradientUnits="userSpaceOnUse" x1="0" y1="56.6904" x2="0.7071" y2="55.9833">
                                        <stop  offset="0" style="stop-color:#259E2E"/>
                                        <stop  offset="0.9831" style="stop-color:#1C381D"/>
                        </linearGradient>
                        <linearGradient id="SVGID_2_" gradientUnits="userSpaceOnUse" x1="-159.6894" y1="-141.4349" x2="-158.2339" y2="-141.4349" gradientTransform="matrix(0 26.9963 -0.8337 0 -89.6816 4320.7969)">
                                        <stop  offset="0" style="stop-color:#1B1F46"/>
                                        <stop  offset="0.3515" style="stop-color:#1B1F46"/>
                                        <stop  offset="0.4737" style="stop-color:#2E467E"/>
                                        <stop  offset="0.5515" style="stop-color:#3665A9"/>
                                        <stop  offset="0.5925" style="stop-color:#3665A8"/>
                                        <stop  offset="0.6315" style="stop-color:#3565A2"/>
                                        <stop  offset="0.6697" style="stop-color:#336699"/>
                                        <stop  offset="0.7074" style="stop-color:#30668C"/>
                                        <stop  offset="0.7448" style="stop-color:#2D667A"/>
                                        <stop  offset="0.7819" style="stop-color:#296663"/>
                                        <stop  offset="0.8181" style="stop-color:#256449"/>
                                        <stop  offset="0.8485" style="stop-color:#216230"/>
                                        <stop  offset="1" style="stop-color:#216230"/>
                        </linearGradient>
                        <rect x="1.6" y="9.8" class="st0" width="53.3" height="39.3"/>
                        <path class="st1" d="M17.9,28.5h11.4c-0.1-2.3-0.7-4.6-2.4-6.1c-2.1-1.8-5.7-2.4-9-2.4c-3.4,0-7.1,0.7-9.2,2.6
                                        c-1.8,1.6-2.2,4.3-2.2,6.7c0,2.5,0.8,5.4,2.7,7.1c2.1,1.8,5.5,2.2,8.7,2.2c3.2,0,6.6-0.5,8.6-2.2c2.1-1.7,2.8-4.5,2.8-7.1
                                        c0,0,0,0,0,0H17.9V28.5z"/>
                        <path class="st1" d="M30.1,29.3v8.9h15.9v0c2.3-0.1,4.2-2,4.2-4.4c0-2.4-1.9-4.4-4.2-4.5v0H30.1z"/>
                        <path class="st1" d="M49.9,24.5c0-2.3-1.8-4.1-4.1-4.1c-0.2,0-0.3,0-0.5,0H30.1v8.2h16C48.3,28.4,49.9,26.6,49.9,24.5z"/>
                        </svg>';

        // MASTERCARD
        $icon .= '<svg  width="27" height="27" version="1.1" id="Layer_1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" x="0px" y="0px"
                    viewBox="0 0 504 504" style="enable-background:new 0 0 504 504;" xml:space="preserve">
                <path style="fill:#FFB600;" d="M504,252c0,83.2-67.2,151.2-151.2,151.2c-83.2,0-151.2-68-151.2-151.2l0,0
                    c0-83.2,67.2-151.2,150.4-151.2C436.8,100.8,504,168.8,504,252L504,252z"/>
                <path style="fill:#F7981D;" d="M352.8,100.8c83.2,0,151.2,68,151.2,151.2l0,0c0,83.2-67.2,151.2-151.2,151.2
                    c-83.2,0-151.2-68-151.2-151.2"/>
                <path style="fill:#FF8500;" d="M352.8,100.8c83.2,0,151.2,68,151.2,151.2l0,0c0,83.2-67.2,151.2-151.2,151.2"/>
                <path style="fill:#FF5050;" d="M149.6,100.8C67.2,101.6,0,168.8,0,252s67.2,151.2,151.2,151.2c39.2,0,74.4-15.2,101.6-39.2l0,0l0,0
                    c5.6-4.8,10.4-10.4,15.2-16h-31.2c-4-4.8-8-10.4-11.2-15.2h53.6c3.2-4.8,6.4-10.4,8.8-16h-71.2c-2.4-4.8-4.8-10.4-6.4-16h83.2
                    c4.8-15.2,8-31.2,8-48c0-11.2-1.6-21.6-3.2-32h-92.8c0.8-5.6,2.4-10.4,4-16h83.2c-1.6-5.6-4-11.2-6.4-16H216
                    c2.4-5.6,5.6-10.4,8.8-16h53.6c-3.2-5.6-7.2-11.2-12-16h-29.6c4.8-5.6,9.6-10.4,15.2-15.2c-26.4-24.8-62.4-39.2-101.6-39.2
                    C150.4,100.8,150.4,100.8,149.6,100.8z"/>
                <path style="fill:#E52836;" d="M0,252c0,83.2,67.2,151.2,151.2,151.2c39.2,0,74.4-15.2,101.6-39.2l0,0l0,0
                    c5.6-4.8,10.4-10.4,15.2-16h-31.2c-4-4.8-8-10.4-11.2-15.2h53.6c3.2-4.8,6.4-10.4,8.8-16h-71.2c-2.4-4.8-4.8-10.4-6.4-16h83.2
                    c4.8-15.2,8-31.2,8-48c0-11.2-1.6-21.6-3.2-32h-92.8c0.8-5.6,2.4-10.4,4-16h83.2c-1.6-5.6-4-11.2-6.4-16H216
                    c2.4-5.6,5.6-10.4,8.8-16h53.6c-3.2-5.6-7.2-11.2-12-16h-29.6c4.8-5.6,9.6-10.4,15.2-15.2c-26.4-24.8-62.4-39.2-101.6-39.2h-0.8"/>
                <path style="fill:#CB2026;" d="M151.2,403.2c39.2,0,74.4-15.2,101.6-39.2l0,0l0,0c5.6-4.8,10.4-10.4,15.2-16h-31.2
                    c-4-4.8-8-10.4-11.2-15.2h53.6c3.2-4.8,6.4-10.4,8.8-16h-71.2c-2.4-4.8-4.8-10.4-6.4-16h83.2c4.8-15.2,8-31.2,8-48
                    c0-11.2-1.6-21.6-3.2-32h-92.8c0.8-5.6,2.4-10.4,4-16h83.2c-1.6-5.6-4-11.2-6.4-16H216c2.4-5.6,5.6-10.4,8.8-16h53.6
                    c-3.2-5.6-7.2-11.2-12-16h-29.6c4.8-5.6,9.6-10.4,15.2-15.2c-26.4-24.8-62.4-39.2-101.6-39.2h-0.8"/>
                <g>
                    <path style="fill:#FFFFFF;" d="M204.8,290.4l2.4-13.6c-0.8,0-2.4,0.8-4,0.8c-5.6,0-6.4-3.2-5.6-4.8l4.8-28h8.8l2.4-15.2h-8l1.6-9.6
                        h-16c0,0-9.6,52.8-9.6,59.2c0,9.6,5.6,13.6,12.8,13.6C199.2,292.8,203.2,291.2,204.8,290.4z"/>
                    <path style="fill:#FFFFFF;" d="M210.4,264.8c0,22.4,15.2,28,28,28c12,0,16.8-2.4,16.8-2.4l3.2-15.2c0,0-8.8,4-16.8,4
                        c-17.6,0-14.4-12.8-14.4-12.8H260c0,0,2.4-10.4,2.4-14.4c0-10.4-5.6-23.2-23.2-23.2C222.4,227.2,210.4,244.8,210.4,264.8z
                        M238.4,241.6c8.8,0,7.2,10.4,7.2,11.2H228C228,252,229.6,241.6,238.4,241.6z"/>
                    <path style="fill:#FFFFFF;" d="M340,290.4l3.2-17.6c0,0-8,4-13.6,4c-11.2,0-16-8.8-16-18.4c0-19.2,9.6-29.6,20.8-29.6
                        c8,0,14.4,4.8,14.4,4.8l2.4-16.8c0,0-9.6-4-18.4-4c-18.4,0-36.8,16-36.8,46.4c0,20,9.6,33.6,28.8,33.6
                        C331.2,292.8,340,290.4,340,290.4z"/>
                    <path style="fill:#FFFFFF;" d="M116.8,227.2c-11.2,0-19.2,3.2-19.2,3.2L95.2,244c0,0,7.2-3.2,17.6-3.2c5.6,0,10.4,0.8,10.4,5.6
                        c0,3.2-0.8,4-0.8,4s-4.8,0-7.2,0c-13.6,0-28.8,5.6-28.8,24c0,14.4,9.6,17.6,15.2,17.6c11.2,0,16-7.2,16.8-7.2l-0.8,6.4H132l6.4-44
                        C138.4,228,122.4,227.2,116.8,227.2z M120,263.2c0,2.4-1.6,15.2-11.2,15.2c-4.8,0-6.4-4-6.4-6.4c0-4,2.4-9.6,14.4-9.6
                        C119.2,263.2,120,263.2,120,263.2z"/>
                    <path style="fill:#FFFFFF;" d="M153.6,292c4,0,24,0.8,24-20.8c0-20-19.2-16-19.2-24c0-4,3.2-5.6,8.8-5.6c2.4,0,11.2,0.8,11.2,0.8
                        l2.4-14.4c0,0-5.6-1.6-15.2-1.6c-12,0-24,4.8-24,20.8c0,18.4,20,16.8,20,24c0,4.8-5.6,5.6-9.6,5.6c-7.2,0-14.4-2.4-14.4-2.4
                        l-2.4,14.4C136,290.4,140,292,153.6,292z"/>
                    <path style="fill:#FFFFFF;" d="M472.8,214.4l-3.2,21.6c0,0-6.4-8-15.2-8c-14.4,0-27.2,17.6-27.2,38.4c0,12.8,6.4,26.4,20,26.4
                        c9.6,0,15.2-6.4,15.2-6.4l-0.8,5.6h16l12-76.8L472.8,214.4z M465.6,256.8c0,8.8-4,20-12.8,20c-5.6,0-8.8-4.8-8.8-12.8
                        c0-12.8,5.6-20.8,12.8-20.8C462.4,243.2,465.6,247.2,465.6,256.8z"/>
                    <path style="fill:#FFFFFF;" d="M29.6,291.2l9.6-57.6l1.6,57.6H52l20.8-57.6L64,291.2h16.8l12.8-76.8H67.2l-16,47.2l-0.8-47.2H27.2
                        l-12.8,76.8H29.6z"/>
                    <path style="fill:#FFFFFF;" d="M277.6,291.2c4.8-26.4,5.6-48,16.8-44c1.6-10.4,4-14.4,5.6-18.4c0,0-0.8,0-3.2,0
                        c-7.2,0-12.8,9.6-12.8,9.6l1.6-8.8h-15.2L260,292h17.6V291.2z"/>
                    <path style="fill:#FFFFFF;" d="M376.8,227.2c-11.2,0-19.2,3.2-19.2,3.2l-2.4,13.6c0,0,7.2-3.2,17.6-3.2c5.6,0,10.4,0.8,10.4,5.6
                        c0,3.2-0.8,4-0.8,4s-4.8,0-7.2,0c-13.6,0-28.8,5.6-28.8,24c0,14.4,9.6,17.6,15.2,17.6c11.2,0,16-7.2,16.8-7.2l-0.8,6.4H392l6.4-44
                        C399.2,228,382.4,227.2,376.8,227.2z M380.8,263.2c0,2.4-1.6,15.2-11.2,15.2c-4.8,0-6.4-4-6.4-6.4c0-4,2.4-9.6,14.4-9.6
                        C380,263.2,380,263.2,380.8,263.2z"/>
                    <path style="fill:#FFFFFF;" d="M412,291.2c4.8-26.4,5.6-48,16.8-44c1.6-10.4,4-14.4,5.6-18.4c0,0-0.8,0-3.2,0
                        c-7.2,0-12.8,9.6-12.8,9.6l1.6-8.8h-15.2L394.4,292H412V291.2z"/>
                </g>
                <g>
                    <path style="fill:#DCE5E5;" d="M180,279.2c0,9.6,5.6,13.6,12.8,13.6c5.6,0,10.4-1.6,12-2.4l2.4-13.6c-0.8,0-2.4,0.8-4,0.8
                        c-5.6,0-6.4-3.2-5.6-4.8l4.8-28h8.8l2.4-15.2h-8l1.6-9.6"/>
                    <path style="fill:#DCE5E5;" d="M218.4,264.8c0,22.4,7.2,28,20,28c12,0,16.8-2.4,16.8-2.4l3.2-15.2c0,0-8.8,4-16.8,4
                        c-17.6,0-14.4-12.8-14.4-12.8H260c0,0,2.4-10.4,2.4-14.4c0-10.4-5.6-23.2-23.2-23.2C222.4,227.2,218.4,244.8,218.4,264.8z
                        M238.4,241.6c8.8,0,10.4,10.4,10.4,11.2H228C228,252,229.6,241.6,238.4,241.6z"/>
                    <path style="fill:#DCE5E5;" d="M340,290.4l3.2-17.6c0,0-8,4-13.6,4c-11.2,0-16-8.8-16-18.4c0-19.2,9.6-29.6,20.8-29.6
                        c8,0,14.4,4.8,14.4,4.8l2.4-16.8c0,0-9.6-4-18.4-4c-18.4,0-28.8,16-28.8,46.4c0,20,1.6,33.6,20.8,33.6
                        C331.2,292.8,340,290.4,340,290.4z"/>
                    <path style="fill:#DCE5E5;" d="M95.2,244.8c0,0,7.2-3.2,17.6-3.2c5.6,0,10.4,0.8,10.4,5.6c0,3.2-0.8,4-0.8,4s-4.8,0-7.2,0
                        c-13.6,0-28.8,5.6-28.8,24c0,14.4,9.6,17.6,15.2,17.6c11.2,0,16-7.2,16.8-7.2l-0.8,6.4H132l6.4-44c0-18.4-16-19.2-22.4-19.2
                        M128,263.2c0,2.4-9.6,15.2-19.2,15.2c-4.8,0-6.4-4-6.4-6.4c0-4,2.4-9.6,14.4-9.6C119.2,263.2,128,263.2,128,263.2z"/>
                    <path style="fill:#DCE5E5;" d="M136,290.4c0,0,4.8,1.6,18.4,1.6c4,0,24,0.8,24-20.8c0-20-19.2-16-19.2-24c0-4,3.2-5.6,8.8-5.6
                        c2.4,0,11.2,0.8,11.2,0.8l2.4-14.4c0,0-5.6-1.6-15.2-1.6c-12,0-16,4.8-16,20.8c0,18.4,12,16.8,12,24c0,4.8-5.6,5.6-9.6,5.6"/>
                    <path style="fill:#DCE5E5;" d="M469.6,236c0,0-6.4-8-15.2-8c-14.4,0-19.2,17.6-19.2,38.4c0,12.8-1.6,26.4,12,26.4
                        c9.6,0,15.2-6.4,15.2-6.4l-0.8,5.6h16l12-76.8 M468.8,256.8c0,8.8-7.2,20-16,20c-5.6,0-8.8-4.8-8.8-12.8c0-12.8,5.6-20.8,12.8-20.8
                        C462.4,243.2,468.8,247.2,468.8,256.8z"/>
                    <path style="fill:#DCE5E5;" d="M29.6,291.2l9.6-57.6l1.6,57.6H52l20.8-57.6L64,291.2h16.8l12.8-76.8h-20l-22.4,47.2l-0.8-47.2h-8.8
                        l-27.2,76.8H29.6z"/>
                    <path style="fill:#DCE5E5;" d="M260.8,291.2h16.8c4.8-26.4,5.6-48,16.8-44c1.6-10.4,4-14.4,5.6-18.4c0,0-0.8,0-3.2,0
                        c-7.2,0-12.8,9.6-12.8,9.6l1.6-8.8"/>
                    <path style="fill:#DCE5E5;" d="M355.2,244.8c0,0,7.2-3.2,17.6-3.2c5.6,0,10.4,0.8,10.4,5.6c0,3.2-0.8,4-0.8,4s-4.8,0-7.2,0
                        c-13.6,0-28.8,5.6-28.8,24c0,14.4,9.6,17.6,15.2,17.6c11.2,0,16-7.2,16.8-7.2l-0.8,6.4H392l6.4-44c0-18.4-16-19.2-22.4-19.2
                        M388,263.2c0,2.4-9.6,15.2-19.2,15.2c-4.8,0-6.4-4-6.4-6.4c0-4,2.4-9.6,14.4-9.6C380,263.2,388,263.2,388,263.2z"/>
                    <path style="fill:#DCE5E5;" d="M395.2,291.2H412c4.8-26.4,5.6-48,16.8-44c1.6-10.4,4-14.4,5.6-18.4c0,0-0.8,0-3.2,0
                        c-7.2,0-12.8,9.6-12.8,9.6l1.6-8.8"/>
                </g>
                <g>
                </g>
                <g>
                </g>
                <g>
                </g>
                <g>
                </g>
                <g>
                </g>
                <g>
                </g>
                <g>
                </g>
                <g>
                </g>
                <g>
                </g>
                <g>
                </g>
                <g>
                </g>
                <g>
                </g>
                <g>
                </g>
                <g>
                </g>
                <g>
                </g>
                </svg>';

        // VISA ELECTRON
        $icon .= '<svg width="27" height="27" version="1.1" style="margin-left:5px;" id="Calque_1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" x="0px" y="0px"
                        viewBox="0 0 56.7 56.7" style="enable-background:new 0 0 56.7 56.7;" xml:space="preserve">
                        <style type="text/css">
                                        .VoÃ_x00BB_teÂ_x00A0__x002D__x0020_Vert{fill:url(#SVGID_1_);stroke:#FFFFFF;stroke-width:0.25;stroke-miterlimit:1;}
                                        .st0{fill:#FFFFFF;}
                                        .st1{fill-rule:evenodd;clip-rule:evenodd;fill:#00579F;}
                                        .st2{fill-rule:evenodd;clip-rule:evenodd;fill:#FAA61A;}
                                        .st3{fill-rule:evenodd;clip-rule:evenodd;fill:#FFFFFF;}
                        </style>
                        <linearGradient id="SVGID_1_" gradientUnits="userSpaceOnUse" x1="0" y1="56.6904" x2="0.7071" y2="55.9833">
                                        <stop  offset="0" style="stop-color:#20AC4B"/>
                                        <stop  offset="0.9831" style="stop-color:#19361A"/>
                        </linearGradient>
                        <rect x="1.5" y="9.7" class="st0" width="53.3" height="39.4"/>
                        <path class="st1" d="M54.9,41.4l0,7.7H1.6L1.5,9.7l2.3,0v29.1c0,1.4,1.2,2.6,2.6,2.6H54.9L54.9,41.4z"/>
                        <polygon class="st1" points="21.7,33.6 24.1,18.9 27.9,18.9 25.5,33.6 21.7,33.6 "/>
                        <path class="st1" d="M39.2,19.3c-0.7-0.3-1.9-0.6-3.4-0.6c-3.7,0-6.4,2-6.4,4.8c0,2.1,1.9,3.3,3.3,4c1.5,0.7,2,1.2,2,1.8
                                        c0,1-1.2,1.4-2.3,1.4c-1.5,0-2.3-0.2-3.5-0.8l-0.5-0.2L27.9,33c0.9,0.4,2.5,0.8,4.2,0.8c4,0,6.5-2,6.6-5c0-1.7-1-2.9-3.2-4
                                        c-1.3-0.7-2.1-1.1-2.1-1.8c0-0.6,0.7-1.3,2.2-1.3c1.2,0,2.1,0.3,2.8,0.6l0.3,0.2L39.2,19.3L39.2,19.3z"/>
                        <path class="st1" d="M44.2,28.4c0.3-0.8,1.5-4.1,1.5-4.1c0,0,0.3-0.9,0.5-1.4l0.3,1.3c0,0,0.7,3.5,0.9,4.2
                                        C46.8,28.4,45,28.4,44.2,28.4L44.2,28.4z M48.9,18.9h-2.9c-0.9,0-1.6,0.3-2,1.2l-5.6,13.4h4c0,0,0.7-1.8,0.8-2.2c0.4,0,4.3,0,4.8,0
                                        c0.1,0.5,0.5,2.2,0.5,2.2h3.5L48.9,18.9L48.9,18.9z"/>
                        <path class="st1" d="M18.6,18.9l-3.7,10l-0.4-2c-0.7-2.3-2.8-4.9-5.2-6.1l3.4,12.8h4l5.9-14.6H18.6L18.6,18.9z"/>
                        <path class="st2" d="M11.4,18.9H5.3l-0.1,0.3c4.7,1.2,7.9,4.1,9.2,7.6l-1.3-6.7C12.9,19.2,12.2,19,11.4,18.9L11.4,18.9z"/>
                        <path class="st3" d="M23.4,47.3H24l0,0l1.7,0l0.1-0.6l-1.7,0l0.2-1.3l1.3,0l0.1-0.6l-1.3,0l0.2-1.2l1.6,0l0.1-0.6l-2.2,0L23.4,47.3
                                        L23.4,47.3z M29.5,47.3h0.6l0,0l1.7,0l0.1-0.6l-1.7,0l0.2-1.3l1.3,0l0.1-0.6l-1.3,0l0.2-1.2l1.6,0l0.1-0.6l-2.2,0L29.5,47.3
                                        L29.5,47.3z M26.5,47.3h0.6l0,0l1.7,0l0.1-0.6l-1.7,0l0.6-3.8l-0.7,0L26.5,47.3L26.5,47.3z M37.4,47.3H38l0,0l0,0l0.5-3.8l1.4,0
                                        l0.1-0.6l-3.2,0l-0.1,0.6h1.2L37.4,47.3L37.4,47.3z M40.1,47.3l0.6,0h0h0l0.2-1.9l0.3,0l1,1.9H43l-1-1.9c1.2-0.3,1.6-2.5,0.1-2.5
                                        l-1.3,0L40.1,47.3L40.1,47.3z M41.1,44.9h0.7c0.6,0,0.9-1.4,0.2-1.4l-0.8,0L41.1,44.9L41.1,44.9z M46,42.7c1.1,0,1.8,1,1.6,2.3
                                        c-0.1,1.3-1.1,2.3-2.2,2.3c-1.1,0-1.8-1-1.6-2.3C44,43.8,45,42.7,46,42.7L46,42.7z M46,43.3c0.8,0,1.3,0.9,1.2,1.8s-0.8,1.8-1.6,1.8
                                        c-0.8,0-1.3-0.9-1.2-1.8C44.5,44.1,45.2,43.3,46,43.3L46,43.3z M48.3,47.3l0.6,0h0h0l0.5-3.4l1.6,3.4h0.5l0.6-4.4h-0.6l-0.5,3.2
                                        l-1.5-3.2H49L48.3,47.3L48.3,47.3z M34.9,42.7c0.5,0,0.9,0.2,1.2,0.5l-0.3,0.6c-0.2-0.3-0.5-0.5-0.9-0.5c-0.8,0-1.5,0.8-1.6,1.7
                                        c-0.1,0.9,0.4,1.7,1.2,1.7c0.4,0,0.8-0.2,1.1-0.5V47c-0.3,0.2-0.7,0.4-1.1,0.4c-1.1,0-1.9-1-1.7-2.3C32.8,43.8,33.8,42.7,34.9,42.7
                                        L34.9,42.7z"/>
                        </svg>
                        ';

        // MAESTRO
        $icon .= '<svg width="27" height="27" version="1.1" id="Layer_1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" x="0px" y="0px"
                        viewBox="0 0 504 504" style="enable-background:new 0 0 504 504;" xml:space="preserve">
                    <g>
                        <path style="fill:#07A9E2;" d="M252,364c30.4-27.2,49.6-67.2,49.6-112s-19.2-84-49.6-112c-26.4-24-62.4-39.2-100.8-39.2
                            C68,100.8,0,168.8,0,252s68,150.4,151.2,150.4C189.6,403.2,225.6,388,252,364L252,364z"/>
                        <path style="fill:#009DDD;" d="M0,252c0,83.2,68,150.4,151.2,150.4c39.2,0,74.4-14.4,100.8-39.2l0,0c30.4-27.2,49.6-67.2,49.6-112
                            S282.4,168,252,140c-26.4-24-62.4-39.2-100.8-39.2"/>
                        <path style="fill:#0284CC;" d="M252,140c-26.4-24-62.4-39.2-100.8-39.2c-2.4,0-4.8,0-7.2,0C96.8,135.2,66.4,190.4,66.4,252
                            s30.4,116.8,76.8,150.4c2.4,0,4.8,0,7.2,0c39.2,0,74.4-14.4,100.8-39.2c30.4-27.2,49.6-67.2,49.6-112S282.4,168,252,140z"/>
                        <path style="fill:#FF2957;" d="M353.6,100.8c-39.2,0-74.4,14.4-101.6,39.2c-5.6,4.8-10.4,10.4-15.2,16h30.4c4,4.8,8,10.4,11.2,16
                            h-53.6c-3.2,5.6-6.4,10.4-8.8,16h71.2c2.4,5.6,4.8,10.4,6.4,16h-83.2c-1.6,5.6-3.2,10.4-4.8,16h92.8c2.4,10.4,3.2,20.8,3.2,32
                            c0,16.8-2.4,32.8-8,48h-83.2c1.6,5.6,4,10.4,6.4,16H288c-2.4,5.6-5.6,11.2-8.8,16h-53.6c3.2,5.6,7.2,11.2,11.2,16h30.4
                            c-4.8,5.6-9.6,11.2-15.2,16c27.2,24,62.4,39.2,101.6,39.2c83.2,0,150.4-67.2,150.4-150.4C504,168.8,436.8,100.8,353.6,100.8
                            L353.6,100.8z"/>
                        <path style="fill:#ED1C2E;" d="M353.6,100.8L353.6,100.8c-39.2,0-74.4,14.4-101.6,39.2c-5.6,4.8-10.4,10.4-15.2,16h30.4
                            c4,4.8,8,10.4,11.2,16h-53.6c-3.2,5.6-6.4,10.4-8.8,16h71.2c2.4,5.6,4.8,10.4,6.4,16h-83.2c-1.6,5.6-3.2,10.4-4.8,16h92.8
                            c2.4,10.4,3.2,20.8,3.2,32c0,16.8-2.4,32.8-8,48h-83.2c1.6,5.6,4,10.4,6.4,16H288c-2.4,5.6-5.6,11.2-8.8,16h-53.6
                            c3.2,5.6,7.2,11.2,11.2,16h30.4c-4.8,5.6-9.6,11.2-15.2,16c27.2,24,62.4,39.2,101.6,39.2c83.2,0,150.4-67.2,150.4-150.4"/>
                        <path style="fill:#0466B5;" d="M252,364c30.4-27.2,49.6-67.2,49.6-112s-19.2-84-49.6-112V364L252,364z"/>
                        <path style="fill:#FFFFFF;" d="M452,284.8c0-2.4,2.4-4.8,4.8-4.8s4.8,2.4,4.8,4.8s-2.4,4.8-4.8,4.8
                            C454.4,289.6,452,287.2,452,284.8L452,284.8z M456.8,288.8c2.4,0,3.2-1.6,3.2-4s-1.6-3.2-3.2-3.2c-2.4,0-4,1.6-4,3.2
                            S455.2,288.8,456.8,288.8L456.8,288.8z M456.8,286.4H456v-4h1.6c0.8,0,0.8,0,0.8,0l0.8,0.8c0,0.8,0,0.8-0.8,0.8l0.8,1.6h-0.8
                            l-0.8-1.6h-0.8L456.8,286.4L456.8,286.4z M456.8,284L456.8,284c0.8,0,0.8,0,0.8,0s0,0,0-0.8l0,0h-0.8l0,0V284L456.8,284z"/>
                        <path style="fill:#D1094B;" d="M252,140c-5.6,4.8-10.4,10.4-15.2,16h30.4c4,4.8,8,10.4,11.2,16h-53.6c-3.2,5.6-6.4,10.4-8.8,16
                            h71.2c2.4,5.6,4.8,10.4,6.4,16h-83.2c-1.6,5.6-3.2,10.4-4.8,16h92.8c2.4,10.4,3.2,20.8,3.2,32c0,16.8-2.4,32.8-8,48h-83.2
                            c1.6,5.6,4,10.4,6.4,16H288c-2.4,5.6-5.6,11.2-8.8,16h-53.6c3.2,5.6,7.2,11.2,11.2,16h30.4c-4.8,5.6-9.6,11.2-15.2,16
                            c27.2,24,62.4,39.2,101.6,39.2c2.4,0,4.8,0,7.2,0c46.4-33.6,76.8-88.8,76.8-150.4S407.2,136,360.8,102.4c-2.4,0-4.8,0-7.2,0
                            C314.4,100.8,279.2,116,252,140z"/>
                        <polygon style="fill:#FFFFFF;" points="132,289.6 112.8,289.6 124,231.2 98.4,289.6 81.6,289.6 78.4,231.2 67.2,289.6 50.4,289.6 
                            64.8,212.8 94.4,212.8 96,260.8 116.8,212.8 147.2,212.8 	"/>
                        <g>
                            <path style="fill:#DCE5E5;" d="M113.6,256l11.2-24.8l-8.8,46.4c0.8,4,1.6,8.8,3.2,12.8H132l14.4-76.8h-28
                                c-3.2,12-5.6,25.6-5.6,39.2C113.6,253.6,113.6,254.4,113.6,256z"/>
                            <path style="fill:#DCE5E5;" d="M341.6,288.8c-5.6,1.6-8.8,2.4-13.6,2.4c-9.6,0-14.4-4.8-14.4-13.6c0-1.6,0-3.2,0.8-5.6l0.8-6.4
                                l0.8-4.8l8.8-47.2h18.4l-2.4,14.4h9.6l-2.4,14.4h-9.6l-4.8,25.6c0,0.8-0.8,2.4-0.8,2.4c0,3.2,2.4,4.8,6.4,4.8c2.4,0,4,0,4.8-0.8
                                L341.6,288.8L341.6,288.8z"/>
                            <path style="fill:#DCE5E5;" d="M396.8,228c-1.6-0.8-1.6-0.8-2.4-0.8s-1.6,0-1.6-0.8c-0.8,0-0.8,0-2.4,0c-6.4,0-10.4,2.4-16,10.4
                                l1.6-9.6h-16.8L348,289.6h19.2c6.4-38.4,9.6-44.8,18.4-44.8c0.8,0,1.6,0,2.4,0l2.4,0.8L396.8,228L396.8,228z"/>
                            <path style="fill:#DCE5E5;" d="M267.2,247.2c0,8,4.8,13.6,14.4,16.8c8,3.2,8.8,4,8.8,7.2c0,4-3.2,5.6-11.2,5.6
                                c-5.6,0-11.2-0.8-16.8-2.4l-2.4,14.4h0.8l3.2,0.8c0.8,0,3.2,0,4.8,0c4.8,0.8,8,0.8,10.4,0.8c20,0,29.6-6.4,29.6-20.8
                                c0-8.8-4-13.6-13.6-17.6c-8-3.2-8.8-4-8.8-6.4c0-3.2,3.2-4.8,9.6-4.8c3.2,0,8.8,0,13.6,0.8l2.4-14.4c-4.8-0.8-12-0.8-16.8-0.8
                                C274.4,226.4,267.2,236,267.2,247.2L267.2,247.2z"/>
                            <path style="fill:#DCE5E5;" d="M192.8,289.6h-15.2v-6.4c-4.8,4.8-11.2,7.2-20,7.2c-10.4,0-16.8-7.2-16.8-16.8
                                c0-15.2,12-24,32.8-24c2.4,0,4.8,0,8,0c0.8-2.4,0.8-3.2,0.8-4c0-4-3.2-5.6-12-5.6c-5.6,0-11.2,0.8-15.2,1.6l-2.4,0.8h-1.6
                                l2.4-13.6c9.6-2.4,15.2-3.2,22.4-3.2c16,0,24.8,6.4,24.8,18.4c0,3.2,0,5.6-1.6,12.8l-4,22.4l-0.8,4l-0.8,3.2v2.4L192.8,289.6
                                L192.8,289.6z M179.2,261.6c-2.4,0-3.2,0-4,0c-10.4,0-16,3.2-16,9.6c0,4,2.4,6.4,6.4,6.4C173.6,277.6,179.2,271.2,179.2,261.6
                                L179.2,261.6z"/>
                            <path style="fill:#DCE5E5;" d="M254.4,288c-6.4,1.6-12.8,2.4-19.2,2.4c-20.8,0-32-9.6-32-28c0-21.6,13.6-36.8,32.8-36.8
                                c15.2,0,24.8,8.8,24.8,22.4c0,4.8-0.8,8.8-2.4,15.2h-37.6c0,0.8,0,1.6,0,2.4c0,7.2,5.6,11.2,16,11.2c7.2,0,12.8-0.8,19.2-4
                                L254.4,288L254.4,288z M244.8,251.2c0-0.8,0-2.4,0-3.2c0-4.8-3.2-8-8.8-8s-10.4,4-12,11.2H244.8L244.8,251.2z"/>
                        </g>
                        <g>
                            <path style="fill:#FFFFFF;" d="M455.2,261.6c-3.2,20.8-17.6,29.6-36.8,29.6c-20.8,0-29.6-12.8-29.6-28.8
                                c0-21.6,14.4-36.8,36.8-36.8c19.2,0,29.6,12.8,29.6,28C455.2,257.6,455.2,257.6,455.2,261.6L455.2,261.6z M435.2,253.6
                                c0-6.4-2.4-12-10.4-12c-9.6,0-15.2,11.2-15.2,21.6c0,8,4,13.6,10.4,13.6c4,0,12.8-5.6,14.4-15.2
                                C435.2,258.4,435.2,256,435.2,253.6L435.2,253.6z"/>
                            <path style="fill:#FFFFFF;" d="M389.6,269.6c0.8-5.6,1.6-12,1.6-17.6c0-0.8,0-1.6,0-1.6c-0.8,4-1.6,8-1.6,12.8
                                C388.8,264.8,388.8,267.2,389.6,269.6z"/>
                        </g>
                    </g>
                    <g>
                    </g>
                    <g>
                    </g>
                    <g>
                    </g>
                    <g>
                    </g>
                    <g>
                    </g>
                    <g>
                    </g>
                    <g>
                    </g>
                    <g>
                    </g>
                    <g>
                    </g>
                    <g>
                    </g>
                    <g>
                    </g>
                    <g>
                    </g>
                    <g>
                    </g>
                    <g>
                    </g>
                    <g>
                    </g>
                    </svg>
                    </div>';


        return apply_filters( 'woocommerce_gateway_icon', $icon, $this->id );
    }

    /**
     *
     * This method contains the custom form for payment on the checkout page.
     *
     * @since    1.0.0
     */
    public function payment_fields()
    {
        try {
            if ($this->get_option( 'mode_test' ) === 'yes'){
                wp_enqueue_script( 'woocommerce_cpa_validation', "https://test-js.centralpay.net/js/validation.js", array(), null, false );
            }else{
                wp_enqueue_script( 'woocommerce_cpa_validation', "https://js.centralpay.net/js/validation.js", array(), null, false );
            }
            // Passer cette partie de la fonction dans la classe DB
            $current_user = wp_get_current_user( );
            $wpUserId = (int)$current_user->ID;
            $rows = DB::getCards($wpUserId, $this->_dbuser, $this->_dbpassword);



            $formSelectedCardvisibility = (string)(!empty($rows)) ? "initial" : "none";
            $formNewCardvisibility = (string)(!empty($rows)) ? "none" : "initial";
            $formNewCardChecked = (string)(!empty($rows)) ? "" : "checked";
            $formSelectedCardChecked = (string)(!empty($rows)) ? "checked" : "";
            $html = '';



            if ($this->testmode) {
                $html .= '<p style="background: none repeat scroll 0 0 #FFFFE0; border: 1px solid #E6DB55; font-size: 13px; margin: 0 0 10px; padding: 10px;">'
                    .    __( 'You are in test mode', 'cpay' )
                    .    '</p>';
            }

            $html .= '<div id="cpay-payment-data modal-content animate">
                        <ul class="wc_payment_methods payment_methods methods">';

            $html .= (string)$this->getFormSelectedCard($rows, $formSelectedCardvisibility, $formSelectedCardChecked);
            $html .= '<li class="wc_payment_method">
                            <input id="payment_method_cpay_guest"
                                    type="radio"
                                    name="selected_card"
                                    value="false" onClick="onChangeRadio()" ' . $formNewCardChecked .'>
                            <label for="payment_method_cpay_guest">';
            $html .= __( 'Pay with a new card', 'cpay' );
            $html .= '</label>';
            $html .= (string)$this->getFormNewCard($formNewCardvisibility);
            $html .= '</li></ul></div>';
            echo $html;

        } catch (Exception $ex) {
            throw new Exception($ex->getMessage());
        }
    }

    /**
     *
     * This function allow you to get the guest form
     *
     * @since    1.0.0
     */
    private function getFormNewCard($formNewCardvisibility)
    {
        if (!isset($formNewCardvisibility)) {
            throw new Exception("Une erreur est survenue lors de la récupération du formulaire de paiement avec une nouvelle carte. ArgumentNullException");
        } else {
            try {
                include_once (dirname(__FILE__).'/templates/formNewCard.php' );
            } catch (Exception $e) {
                throw new Exception( $e );
            }
        }
    }

    /**
     *
     * Get the authenticated form when a wordpress user is identified
     *
     * @since    1.0.0
     */
    private function getFormSelectedCard($rows, $visibility, $formSelectedCardchecked)
    {
        if (!isset($rows) || !isset($visibility) || !isset($formSelectedCardchecked)) {
            throw new Exception("Une erreur est survenue lors de la récupération du formulaire de sélection des cartes de crédits ArgumentNullException.");
        } else {
            try {
                $display = "";
                if (empty($rows)){ $display = "display:none"; }
                $form = '<li class="wc_payment_method"  style="'.$display.'">
                                <input id="payment_method_cpay_authenticated"
                                        type="radio"
                                        name="selected_card"
                                        value="false" onClick="onChangeRadio()"' . $formSelectedCardchecked . '/>';

                if (!empty($rows)) {
                    $form .= '<label for="payment_method_cpay_authenticated">' . __( 'Select your payment card', 'cpay' ) . '</label>
                          <div id="payment_box_withCard" style="display: '. $visibility . ';"><select  name="select" >';

                    foreach ($rows as $obj) :
                        $cardType = '';

                        // Logo don't work at this time' ....
                        $LogoCardStyleBackground = '';
                        $firstChar = substr($obj->first6, 0, -5);

                        if ($firstChar === '4') {
                            $cardType = 'Visa';
                        }

                        if ($firstChar === '5') {
                            $cardType = 'Mastercard';
                        }
                        // var_dump($obj->default);die();

                        $selected = ($obj->default == 1) ? 'selected="selected"' : '';

                        $form .= '<option class="optionImg" value="' . $obj->cardId . '" '.$selected.' style="' . $LogoCardStyleBackground  . '">'
                            . '' . substr($obj->first6, 0, -5) . '***-****-****-**' . substr($obj->last4, 2, 2)
                            . ' | Expiration: ' . $obj->expirationMonth  . ' / ' . $obj->expirationYear
                            . ' ' . $cardType . '</option>';
                    endforeach;

                    $form .= '</select>
                        <div class="input-half input-half-last">
                                    <label class="sr-only" for="customerValidationValue"><span>CVC</span></label>
                                    <div class="input-holder">
                                        <span class="txt_dyn fa fa-lock"></span>
                                        <input inputmode="numeric" 
                                                id="customerValidationValue" 
                                                autocomplete="off"
                                                autocorrect="no" 
                                                autocapitalize="no"
                                                spellcheck="no"
                                                type="tel"
                                                maxlength="3" 
                                                size="3"
                                                name="customerValidationValue" 
                                                pattern="/^[0-9]{3,4}$/"
                                                placeholder="' . __( 'CVC', 'cpay' ) . '">
                                        <div class="help-tip">
                                            <p>Les 3 chiffres au dos de votre carte.</p>
                                        </div>
                                    </div>
                                </div>
                            </div>';
                }
                $form .= "</li>";

                return $form;
            } catch (PDOException $Exception) {
                throw new PDOException( $Exception->getMessage( ), (int)$Exception->getCode( ) );
            } catch (Exception $e) {
                throw new Exception( $e->getMessage( ) );
            }
        }
    }

    /**
     * You can get the payment data
     *
     * @return array
     */
    private function getPaymentData($currentUserId, $order)
    {
        try {
            return array(
                'userId' =>                             $currentUserId,
                'order' =>                              $order,
                'isSavedCard' =>                        (!empty($_POST["save_account"])) ? ($_POST['save_account'] === 'true'? true: false) : false,
                'isSelectedCard' =>                     (!empty($_POST["selected_card"])) ? ($_POST['selected_card'] === 'true'? true: false) : false,
                'currency' =>                           (!empty($_POST["currency"])) ? $_POST['currency'] : "",
                'amount' =>                             ($order->get_total() * 100),
 //               'amount' =>                             ($order->order_total * 100),
                'cardTokenId' =>                        (!empty($_POST["cardTokenId"])) ? $_POST['cardTokenId'] : "",
                'firstName' =>                          (!empty($_POST["billing_first_name"])) ? $_POST['billing_first_name'] : "",
                'lastName' =>                           (!empty($_POST["billing_last_name"])) ? $_POST['billing_last_name'] : "",
                'billing_company' =>                    (!empty($_POST["billing_company"])) ? $_POST['billing_company'] : "",
                'billing_country' =>                    (!empty($_POST["billing_country"])) ? $_POST['billing_country'] : "",
                'billing_address_1' =>                  (!empty($_POST["billing_address_1"])) ? $_POST['billing_address_1'] : "",
                'billing_address_2' =>                  (!empty($_POST["billing_address_2"])) ? $_POST['billing_address_2'] : "",
                'billing_city' =>                       (!empty($_POST["billing_city"])) ? $_POST['billing_city'] : "",
                'billing_state' =>                      (!empty($_POST["billing_state"])) ? $_POST['billing_state'] : "",
                'billing_postcode' =>                   (!empty($_POST["billing_postcode"])) ? $_POST['billing_postcode'] : "",
                'billing_phone' =>                      (!empty($_POST["billing_phone"])) ? $_POST['billing_phone'] : "",
                'billing_email' =>                      (!empty($_POST["billing_email"])) ? $_POST['billing_email'] : "",
                'createaccount' =>                      (!empty($_POST["createaccount"])) ? $_POST['createaccount'] : "false",
                'number' =>                             (!empty($_POST["number"])) ? $_POST['number'] : "",
                'expirationMonth' =>                    (!empty($_POST["expiration"])) ? substr($_POST['expiration'], 0, -4) : "",
                'expirationYear' =>                     (!empty($_POST["expiration"])) ? substr($_POST['expiration'], -2) : "",
                'holderEmail' =>                        (!empty($_POST["holderEmail"])) ? $_POST['holderEmail'] : "",
                'holderName' =>                         (!empty($_POST["holderName"])) ? $_POST['holderName'] : "",
                'first6' =>                             (!empty($_POST["first6"])) ? $_POST['first6'] : '',
                'last4' =>                              (!empty($_POST["last4"])) ? $_POST['last4'] : '',
                'customerValidationValue' =>            (!empty($_POST["customerValidationValue"])) ? $_POST['customerValidationValue'] : '',
                'endUserIp' =>                          $this->testmode ? CpayConfig::CPAY_API_END_USER_ID : ((!empty($_SERVER['REMOTE_ADDR'])) ? $_SERVER['REMOTE_ADDR'] : ""),
                'userPassword' =>                       ''.$this->login_cpay.':'.$this->password_cpay.'',
                'pointOfSaleId' =>                      ''.$this->point_of_sale_id.'',
                'customerId' =>                         DB::getCustomerId($currentUserId, $this->_dbuser, $this->_dbpassword),
                'cardId' =>                             (!empty($_POST["cardId"]) ? $_POST["cardId"] : ""),
                'dateCreation' =>                       '',
//                'merchantTransactionId' =>              $order->id,
                'merchantTransactionId' =>              $order->get_id(),
//                'browserUserAgent' =>                   wc_get_order($order->id)->get_customer_user_agent(),
                'browserUserAgent' =>                   wc_get_order($order->get_id())->get_customer_user_agent(),
                'browserAcceptLanguage' => $_SERVER['HTTP_ACCEPT_LANGUAGE'],
                'endUserLanguage' => substr($_SERVER['HTTP_ACCEPT_LANGUAGE'], 0, 2),
            );
        } catch (Exception $e) {
            throw new Exception( $e->getMessage( ) );
        }
    }

    /**
     * Créer un nouveau customer dans l'api central pay puis dans la base de données wp
     *
     * @return void
     */
    private function cpayCreateCustomer()
    {
        try {

            $urlApi = CpayConfig::CPAY_API_PROD_CUSTOMER_URI;
            if ($this->testmode) {
                $urlApi = CpayConfig::CPAY_API_TEST_CUSTOMER_URI;
            }

            $response = CpayApi::curl_post($urlApi,
                $this->_paymentData['userPassword'],
                array(
                    'cardTokenId' => $this->_paymentData['cardTokenId'],
                    'firstName' => $this->_paymentData['firstName'],
                    'lastName' => $this->_paymentData['lastName'],
                    'addressLine1' => $this->_paymentData['billing_address_1'],
                    'addressLine2' => $this->_paymentData['billing_address_2'],
                    'city' => $this->_paymentData['billing_city'],
                    'postalCode' => $this->_paymentData['billing_postcode'],
                    'phone' => $this->_paymentData['billing_phone'],
                    'email' => $this->_paymentData['billing_email']));
            // var_dump($response);die();
            // 200 OK Created
            if ($response['http_code']  == 200) {
                $responseBody = json_decode($response['body'], true);

                $this->_paymentData['customerId'] = $responseBody['customerId'];
                $this->_paymentData['cardId'] = $responseBody['cards'][0]['cardId'];
                $this->_paymentData['dateCreation'] = date("D M d, Y G:i");

                // CREATE CUSTOMER IN DB
                DB::createCustomer($this->_paymentData, $this->_dbuser, $this->_dbpassword);
            } else { // NOT CREATED
                throw new Exception('Http error during cpay create customer:' . $response['http_code']);
            }
        } catch (Exception $e) {
            throw new Exception( $e->getMessage( ) );
        }
    }

    /**
     * Allow you tu create a new card in the central pay api
     *
     * @return void
     */
    private function createCard($save)
    {
        try {

            $urlApi = CpayConfig::CPAY_API_PROD_CARD_URI;
            if ($this->testmode) {
                $urlApi = CpayConfig::CPAY_API_TEST_CARD_URI;
            }

            $response = CpayApi::curl_post($urlApi,
                $this->_paymentData['userPassword'],
                array(
                    'customerId' => $this->_paymentData['customerId'],
                    'cardTokenId' => $this->_paymentData['cardTokenId'],
                ));

            if ($response['http_code']  == 200) {
                $responseBody = json_decode( $response['body'], true );
                $this->_paymentData['cardId'] = $responseBody['cardId'];
                // CREATE CARD IN DB
                if($save){
                    DB::createCard($this->_paymentData, $this->_dbuser, $this->_dbpassword);
                }
            } else {
                // NOT CREATED
                $this->_paymentData['cardTokenId'] = '';
                throw new Exception( $response['http_code'] . ' during the central pay create card post request.' );
            }
        } catch (Exception $e) {
            throw new Exception( $e->getMessage( ) );
        }
    }

    /**
     * Vérifie si la carte doit être sauvegardé.
     *
     * @return boolean
     */
    private function isSavedCard()
    {
        if ($this->_paymentData['isSelectedCard'] == false && $this->_paymentData['isSavedCard'] == true) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * Vérifie si l'utilisateur est connecté au compte wordpress et au compte central pay.
     *
     * @return boolean
     */
    private function isConnected()
    {
        if (($this->_paymentData['userId'] != 0 && $this->_paymentData['userId'] != '') && ($this->_paymentData['customerId'] != false || $this->_paymentData['customerId'] != '')) {
            return true;
        } else {
            return false;
        }
    }

    /**
     *
     * Process a payment
     *
     * @since    1.0.0
     * @param $order_id This parameter is the order id
     */
    public function process_payment($order_id)
    {
        $this->_paymentData = $this->getPaymentData(get_current_user_id(), wc_get_order( $order_id ));

        $first = false;
//        if ($this->_paymentData['createaccount'] == 0 && $this->_paymentData['isSavedCard'] == 1 && $this->_paymentData['isSelectedCard'] == 0 && get_current_user_id() == 0) {
//            throw new Exception('You have need to check the create account button before save card to account.');
//        }
        // Prepare transaction
        if (is_user_logged_in()) {
            if($this->_paymentData['firstName'] === ''){
                $this->_paymentData['firstName'] = wc_get_order( $order_id )->get_billing_first_name();
            }
            if($this->_paymentData['lastName'] === ''){
                $this->_paymentData['lastName'] = wc_get_order( $order_id )->get_billing_last_name();
            }
        }
        $this->_postvars = array(
            'currency' => $this->_paymentData['currency'],
            'amount' => $this->_paymentData['amount'],
            'endUserIp' => $this->_paymentData['endUserIp'],
            'merchantTransactionId' => $this->_paymentData['merchantTransactionId'],
            'browserUserAgent' => $this->_paymentData['browserUserAgent'],
            'browserAcceptLanguage'=> $this->_paymentData['browserAcceptLanguage'],
            'endUserLanguage'=>$this->_paymentData['endUserLanguage'],
            'order[country]'=>$this->_paymentData['billing_country'],
            'order[firstName]'=> $this->_paymentData['firstName'],
            'order[lastName]'=> $this->_paymentData['lastName'],
        );
        if (is_user_logged_in()) {
            if ($this->_paymentData['customerId'] == false) {
                try{
                    $this->cpayCreateCustomer();
                    $first = true;
                }catch (Exception $e){
                    return $this->paymentError();
                }
            }

            if($this->_paymentData['isSelectedCard'] == true){
                $this->_postvars['cardTokenId'] = '';
                unset($this->_postvars['cardTokenId']);
                $this->_postvars['customerId'] = $this->_paymentData['customerId'];
                $this->_postvars['cardId'] = (!empty($_POST["select"])) ? $_POST['select'] : "";
                $this->_postvars['customerValidationValue'] = (!empty($_POST["customerValidationValue"])) ? $_POST['customerValidationValue'] : "";
            }else{
                if ($this->_paymentData['isSavedCard']) {
                    if(!$first){
                        try{
                            $this->createCard(true);
                        }catch (Exception $e){
                            return $this->paymentError();
                        }
                    }else{
                        DB::createCard($this->_paymentData, $this->_dbuser, $this->_dbpassword);
                    }
                    // Post customerId
                    $this->_postvars['customerId'] = $this->_paymentData['customerId'];
                    $this->_postvars['cardId'] = $this->_paymentData['cardId'];
                    $this->_postvars['cardTokenId'] = '';

                    // var_dump('isSavedCard');die();
                } else {
                    // 2.2 Si la carte n'est pas sauvegardé envoyer le card token id
                    // Post customerId
                    if(!$first){
                        try{
                            $this->createCard(false);
                        }catch (Exception $e){
                            return $this->paymentError();
                        }
                    }
                    $this->_postvars['receiptEmail'] = $this->_paymentData['holderEmail'];
                    $this->_postvars['customerId'] = $this->_paymentData['customerId'];
                    $this->_postvars['cardId'] = $this->_paymentData['cardId'];
                }
            }
        }
        else {
            $this->_postvars['cardTokenId'] = $this->_paymentData['cardTokenId'];
            $this->_postvars['receiptEmail'] = $this->_paymentData['holderEmail'];
        }

        // IF not 3-DS, allow classic curl post
        if ($this->_3ds === false) {
            try {

                $urlApi = CpayConfig::CPAY_API_PROD_TRANSACTION_URI;
                if ($this->testmode) {
                    $urlApi = CpayConfig::CPAY_API_TEST_TRANSACTION_URI;
                }

                $response = CpayApi::curl_post($urlApi, $this->_paymentData['userPassword'], $this->_postvars);
                $order = new WC_Order(  $order_id );
                $url = plugin_dir_url(__FILE__);
                $errorPage = $url."class-cpay-errorForm.php". "?error=" . $response['http_code']. "&paymentPageUrl=" .get_permalink( woocommerce_get_page_id( 'pay' ) );
                switch ($response['http_code']){
                    case 200:
                        $orderStatus = $this->getOrderStatus($order);
                        $order->reduce_order_stock();
                        $order->update_status( $orderStatus );
                        return ['result'   => 'success', 'redirect'=>$this->get_return_url( $this->_paymentData['order'] )];
                    case 402:
                        $order->update_status('failed');
                        return ['result'   => 'success', 'redirect'=>$errorPage];
                    default:
                        $order->update_status('failed');
                        return ['result'   => 'success', 'redirect'=>$errorPage];
                }

            } catch (Exception $e) {
                $order->payment_failed();
                throw new Exception($e->getMessage());
            }
        }else{
            return array(
                'result'   => 'success',
                'redirect' => $this->checkEnrollment($order_id)
            );
        }


    }

    /**
     * Allow you to perform the check enrollment for 3-D Secure.
     *
     * @return void
     */
    private function checkEnrollment($orderId)
    {
        if (!isset($orderId)) {
            throw new Exception("An error was occurred when check enrollment is called orderid is not set.");
        } else {
            try {

                $urlApi = CpayConfig::CPAY_API_PROD_CHECK_ENROLLMENT_URI;
                if ($this->testmode) {
                    $urlApi = CpayConfig::CPAY_API_TEST_CHECK_ENROLLMENT_URI;
                }

                $this->_postvars['merchantTransactionIdentifier'] = "REF001";
                $this->_postvars['merchantName'] = "Marchand 001";
                $this->_postvars['merchantCountry'] = "FRA";
                $this->_postvars['merchantURL'] = "https://www.ma-boutique.com";

                $enrollment = CpayApi::curl_post($urlApi, $this->_paymentData['userPassword'], $this->_postvars);
                $enrollmentBody = json_decode($enrollment['body'], true);

                // status failure is OK donc le checker aussi
                if ($enrollment['http_code']  == 200) {
                    // Param status with failure or success
                    if (! session_id()) {
                        ini_set('session.cookie_samesite', 'None');
                        ini_set('session.cookie_secure', true);
                        @ session_start();
                    }

                    if (! isset($_SESSION['wcpayment'])) {
                        $_SESSION['wcpayment'] = array();
                    }

                    $_SESSION['wcpayment']['userPassword']              = $this->_paymentData['userPassword'];
                    $_SESSION['wcpayment']['currency']                  = $this->_paymentData['currency'];
                    $_SESSION['wcpayment']['amount']                    = $this->_paymentData['amount'];
                    $_SESSION['wcpayment']['customerId']                = $this->_paymentData['customerId'];
                    $_SESSION['wcpayment']['cardId']                    = $this->_paymentData['cardId'];
                    $_SESSION['wcpayment']['cardTokenId']               = $this->_paymentData['cardTokenId'];
                    $_SESSION['wcpayment']['endUserIp']                 = $this->_paymentData['endUserIp'];
                    $_SESSION['wcpayment']['orderId']                   = $orderId;
                    $_SESSION['wcpayment']['redirectUrl']               = $this->get_return_url( $this->_paymentData['order'] );
                    $_SESSION['wcpayment']['paymentPageUrl']            = get_permalink( woocommerce_get_page_id( 'pay' ) );
                    $_SESSION['wcpayment']['isConnected']               = $this->isConnected();
                    $_SESSION['wcpayment']['isSavedCard']  = $this->isSavedCard();
                    $_SESSION['wcpayment']['isSelectedCard'] = $this->_paymentData['isSelectedCard'];
                    $_SESSION['wcpayment']['paymentData'] = $this->_paymentData;
                    $_SESSION['wcpayment']['testmode'] = $this->testmode;

                    $url = plugin_dir_url(__FILE__);
                    // Get the field paReq receive from response ACS
                    $this->_paReq = $enrollmentBody['paReq'];
                    $this->_acsURL = $enrollmentBody['acsURL'];
                    $this->_termUrl = $url."class-cpay-processPaymentCpay.php";
                    $_downloadForm = $url."class-cpay-acsForm.php?paReq=" . urlencode($this->_paReq) . "&acsURL=$this->_acsURL&termUrl=$this->_termUrl";
                    return $_downloadForm;
                } else {
                    throw new Exception('Error during check enrollment return http code ' . $enrollment['http_code']
                        . ' check if your card or your bank accept 3-D Secure payment.');
                }
            } catch (Exception $e) {
                throw new Exception( $e->getMessage( ) );
            }
        }
    }


    private function paymentError($error = 402){
        $url = plugin_dir_url(__FILE__);
        $errorPage = $url."class-cpay-errorForm.php". "?error=".$error."&paymentPageUrl=" .get_permalink( woocommerce_get_page_id( 'pay' ) );
        return ['result'   => 'success', 'redirect'=>$errorPage];
    }

    private function getOrderStatus(WC_Order $order)
    {
        $items = $order->get_items();
        $nbOrderItems = count($items);
        $nbDownloadableItems = 0 ;

        foreach($items as $item)
        {
            $product = $item->get_product();
            $productSimple = $product->is_type('simple');
            $productDownloadable = $product->is_downloadable();
            if($productSimple && $productDownloadable)
            {
                $nbDownloadableItems ++;
            }
        }

        if( $nbDownloadableItems === $nbOrderItems )
        {
            return 'completed';
        }

        return 'processing';
    }
}
