<?php
/**
 * The public-facing functionality of the plugin.
 *
 * @link       http://www.centralpay.eu
 * @since      1.0.0
 *
 * @package    cpay
 * @subpackage cpay/public
 */

/**
 * The public-facing functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    cpay
 * @subpackage cpay/public
 * @author     Central Pay
 */
class CPayPublic
{

    /**
     * The ID of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string    $plugin_name    The ID of this plugin.
     */
    private $plugin_name;

    /**
     * The version of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string    $version    The current version of this plugin.
     */
    private $version;

    /**
     * Initialize the class and set its properties.
     *
     * @since    1.0.0
     * @param      string    $plugin_name       The name of the plugin.
     * @param      string    $version    The version of this plugin.
     */
    public function __construct($plugin_name, $version)
    {

        $this->plugin_name = $plugin_name;
        $this->version = $version;
    }

    /**
     * Register the stylesheets for the public-facing side of the site.
     *
     * @since    1.0.0
     */
    public function enqueueStyles()
    {

        /**
         * This function is provided for demonstration purposes only.
         *
         * An instance of this class should be passed to the run() function
         * defined in Cpay_Loader as all of the hooks are defined
         * in that particular class.
         *
         * The Cpay_Loader will then create the relationship
         * between the defined hooks and the functions defined in this
         * class.
         */

        wp_enqueue_style( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'css/cpay-public.css', array(), $this->version, 'all' );
    }

    /**
     * Register the JavaScript for the public-facing side of the site.
     *
     * @since    1.0.0
     */
    public function enqueueScripts()
    {

        /**
         * This function is provided for demonstration purposes only.
         *
         * An instance of this class should be passed to the run() function
         * defined in Cpay_Loader as all of the hooks are defined
         * in that particular class.
         *
         * The Cpay_Loader will then create the relationship
         * between the defined hooks and the functions defined in this
         * class.
         */

        wp_enqueue_script( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'js/cpay-public.js', array( 'jquery', 'jquery-payment' ), $this->version, false );
    }

    /**
     * Init the gateways.
     *
     * @since    1.0.0
     */
    public function initGateway()
    {
        require_once( dirname( __FILE__ ) . '/includes/class-wc-gateway-cpay.php' );

        add_filter( 'woocommerce_payment_gateways', array( $this, 'add_gateways' ) );
    }
}
